import os
import json
import logging
from beir.retrieval.evaluation import EvaluateRetrieval
from beir.retrieval.search.dense import DenseRetrievalExactSearch as DRES

logger = logging.getLogger(__name__)

class COIR:
    def __init__(self, tasks):
        self.tasks = tasks

    def run(self, model, output_folder: str):
        results = {}
        for task_name, task_data in self.tasks.items():
            corpus, queries, qrels = task_data

            # Initialize custom model
            custom_model = DRES(model, batch_size=128)
            retriever = EvaluateRetrieval(custom_model, score_function="cos_sim")

            # Retrieve results
            task_results = retriever.retrieve(corpus, queries)

            # Evaluate results
            ndcg, _map, recall, precision = retriever.evaluate(qrels, task_results, retriever.k_values)
            metrics = {
                "NDCG": ndcg,
                "MAP": _map,
                "Recall": recall,
                "Precision": precision
            }

            # Save results
            os.makedirs(output_folder, exist_ok=True)
            with open(os.path.join(output_folder, f"{task_name}.json"), 'w') as json_file:
                json.dump({"metrics": metrics}, json_file, indent=4)

            logger.info(f"Results for {task_name} saved to {output_folder}")
            results[task_name] = metrics
        return results


import torch
import numpy as np
import logging
from transformers import AutoTokenizer, AutoModel
from typing import List, Dict
from tqdm.auto import tqdm

logger = logging.getLogger(__name__)

device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

class YourCustomDEModel:
    def __init__(self, model_name="intfloat/e5-base-v2", **kwargs):
        self.tokenizer = AutoTokenizer.from_pretrained(model_name)
        self.model = AutoModel.from_pretrained(model_name).to(device)
        self.model_name = model_name

    def mean_pooling(self, model_output, attention_mask):
        token_embeddings = model_output[0]  # First element of model_output contains all token embeddings
        input_mask_expanded = attention_mask.unsqueeze(-1).expand(token_embeddings.size()).float()
        sum_embeddings = torch.sum(token_embeddings * input_mask_expanded, 1)
        sum_mask = torch.clamp(input_mask_expanded.sum(1), min=1e-9)
        return sum_embeddings / sum_mask

    def encode_text(self, texts: List[str], batch_size: int = 12, max_length: int = 128) -> np.ndarray:
        logging.info(f"Encoding {len(texts)} texts...")

        embeddings = []
        for i in tqdm(range(0, len(texts), batch_size), desc="Encoding batches", unit="batch"):
            batch_texts = texts[i:i+batch_size]
            encoded_input = self.tokenizer(batch_texts, padding=True, truncation=True, max_length=max_length, return_tensors="pt").to(device)
            with torch.no_grad():
                model_output = self.model(**encoded_input)
            batch_embeddings = self.mean_pooling(model_output, encoded_input['attention_mask'])
            embeddings.append(batch_embeddings.cpu())

        embeddings = torch.cat(embeddings, dim=0)

        if embeddings is None:
            logging.error("Embeddings are None.")
        else:
            logging.info(f"Encoded {len(embeddings)} embeddings.")

        return embeddings.numpy()

    def encode_queries(self, queries: List[str], batch_size: int = 12, max_length: int = 512, **kwargs) -> np.ndarray:
        all_queries = ["query: "+ query for query in queries]
        return self.encode_text(all_queries, batch_size, max_length)

    def encode_corpus(self, corpus: List[Dict[str, str]], batch_size: int = 12, max_length: int = 512, **kwargs) -> np.ndarray:
        all_texts = ["passage: "+ doc['text'] for doc in corpus]
        return self.encode_text(all_texts, batch_size, max_length)


import json
import logging
from io import StringIO
from typing import Dict, Tuple
import csv
from datasets import load_dataset
from tqdm.auto import tqdm

logger = logging.getLogger(__name__)

class InMemoryDataLoader:
    def __init__(self, corpus_data, query_data, qrels_data):
        self.corpus_file = StringIO('\n'.join(json.dumps(doc) for doc in corpus_data))
        self.query_file = StringIO('\n'.join(json.dumps(query) for query in query_data))
        self.qrels_file = StringIO('\n'.join(f"{qrel['query_id']}\t{qrel['corpus_id']}\t{qrel['score']}" for qrel in qrels_data))
        self.corpus = {}
        self.queries = {}
        self.qrels = {}

    def load_custom(self) -> Tuple[Dict[str, Dict[str, str]], Dict[str, str], Dict[str, Dict[str, int]]]:
        logger.info("Loading Corpus...")
        self._load_corpus()
        logger.info("Loaded %d Documents.", len(self.corpus))
        logger.info("Doc Example: %s", list(self.corpus.values())[0])

        logger.info("Loading Queries...")
        self._load_queries()

        self._load_qrels()
        self.queries = {qid: self.queries[qid] for qid in self.qrels}
        logger.info("Loaded %d Queries.", len(self.queries))
        logger.info("Query Example: %s", list(self.queries.values())[0])

        return self.corpus, self.queries, self.qrels

    def _load_corpus(self):
        self.corpus_file.seek(0)  # Reset the StringIO object to the beginning
        for line in tqdm(self.corpus_file):
            doc = json.loads(line)
            self.corpus[doc["_id"]] = {
                "text": doc.get("text"),
                "title": doc.get("title")
            }

    def _load_queries(self):
        self.query_file.seek(0)  # Reset the StringIO object to the beginning
        for line in self.query_file:
            query = json.loads(line)
            self.queries[query["_id"]] = query.get("text")

    def _load_qrels(self):
        self.qrels_file.seek(0)  # Reset the StringIO object to the beginning
        reader = csv.reader(self.qrels_file, delimiter="\t", quoting=csv.QUOTE_MINIMAL)
        for row in reader:
            query_id, corpus_id, score = row[0], row[1], int(row[2])
            if query_id not in self.qrels:
                self.qrels[query_id] = {corpus_id: score}
            else:
                self.qrels[query_id][corpus_id] = score

def load_data_from_hf(task_name):
    try:
        queries_corpus_dataset = load_dataset(f"CoIR-Retrieval/{task_name}-queries-corpus")
        qrels_dataset = load_dataset(f"CoIR-Retrieval/{task_name}-qrels")

        corpus_data = queries_corpus_dataset['corpus']
        query_data = queries_corpus_dataset['queries']
        qrels_data = qrels_dataset['test']

        data_loader = InMemoryDataLoader(corpus_data, query_data, qrels_data)
        return data_loader.load_custom()
    except Exception as e:
        logger.error(f"Failed to load data for task {task_name}: {e}")
        return None

def get_tasks(tasks: list):
    all_tasks = {}
    for task in tasks:
        task_data = load_data_from_hf(task)
        if task_data is not None:
            all_tasks[task] = task_data
    return all_tasks


from setuptools import setup, find_packages

setup(
    name="coir-eval",
    version="0.3.0",
    author="xiangyang Li",
    author_email="xiangyangli@pku.edu.cn",
    description="A package for COIR evaluations",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/coir",
    packages=find_packages(exclude=['tests*']),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "datasets>=2.19.0",
        "numpy>=1.0.0",
        "requests>=2.26.0",
        "scikit_learn>=1.0.2",
        "scipy>=0.0.0",
        "sentence_transformers>=3.0.0",
        "torch>1.0.0",
        "tqdm>1.0.0",
        "rich>=0.0.0",
        "pytrec-eval-terrier>=0.5.6",
        "beir"
    ],
)